package DB::Local;

use Carp;
use MLDBM qw(DB_File Storable);
use DB_File;
use Fcntl;
use strict;

## Tie hash for accessing database created with LocalDB.pm

sub TIEHASH {
  my ($class, $id, $dir) = @_;
  my $node = {'id' => $id, 'db' => {}, 'dir' => $dir, 'cache' => {} };
  
  tie %{$node->{'db'}}, 'MLDBM', "$dir/$id",  O_RDONLY, 
   0664, $DB_BTREE or croak "Cannot tie $dir/$id : $!\n";
  
  return bless $node, $class;
}

sub FETCH {
  my $self = shift;
  my $id = shift;
  if (! exists $self->{'cache'}{$id}) {
    return $self->{'cache'}{$id} = $self->{'db'}->{$id};
  } else {
    return $self->{'cache'}{$id};
  }
}

#sub STORE {
#  my $self = shift;
#  my $id = shift;
#  my $value = shift;
#  
#  ## Cannot store data here, do it with LocalDB.pm
#}

sub EXISTS {
  my $self = shift;
  my $id = shift || return 0;

  return (exists $self->{'cache'}->{$id} or exists $self->{'db'}->{$id});
}

#sub DELETE {
#  my $self= shift;
#  my $id = shift;
#
#  #return delete $self->{'db'}->{$id};
#}

#sub CLEAR {
#  my $self = shift;
#
#  #%{$self->{'db'}} = ();
#}

sub FIRSTKEY {
  my $self = shift;
  my $count = scalar keys %{$self->{'db'}}; ## Reset the iterator

  return scalar each %{$self->{'db'}};
}

sub NEXTKEY {
  my $self = shift;

  return scalar each %{$self->{'db'}};
}

sub DESTROY {
  my $self = shift;
  
  %{$self->{'cache'}} = ();
  untie %{$self->{'db'}};
}

## Other specific methods allowed

sub clear_cache {
  my $self = shift;
  %{$self->{'cache'}} = ();
}

## Get the index of all quotes : code => name
## The argument is a reference to a hash
sub get_index {
  my $r_hash = shift;
  my $dir = shift;
  my %hash;

  tie %hash, 'DB_File', "$dir/index", O_RDONLY, 0664, $DB_BTREE
    or croak "Cannot access to $dir/index database !\n";
  %{$r_hash} = %hash;
  untie %hash;
}

1;
