package GTViewWindow;

#
# Viewing window
# draw all indicators and the quotes
#

use strict;
use Conf;
use Tk;
use Carp qw(croak carp);
use Language;
use Calculator;
use GTUtils qw(:date);
use vars qw(@strings %representations);

# Rcupre les textes des messages une fois au dbut pour viter des
# accs rpts  gettext aprs (notamment avec la mise  jour de l'affichage
#  chaque mouvement de souris...)
@strings = (	gettext("Commands"),
		gettext("Opening"),
		gettext("Highest"),
		gettext("Lowest"),
		gettext("Closing"),
		gettext("Volume"),
		gettext("Date"),
		gettext("%y/%m/%d"),
                gettext("Export to postscript"),
		gettext("Close"),
		gettext("Price") );

#
### Quelques explications sur le contenu du hash qui reprsente l'objet :
#
# db         => rfrence sur un hash "lis"  la base de donnes (MLDBM)
# id         => identificateur de la valeur  afficher
# window     => rfrence sur la fentre
# canvas     => rfrence sur un objet Scrolled contenant un canvas
# frame      => rfrence sur la frame qui contient les infos du bas
# height     => hauteur du canvas
# width      => largeur du canvas
# width_per_date => nbre de pixels de largeur par jour
# c_width    => largeur du dessin sur le canvas
# dates      => une rfrence sur un tableau contenant la liste des
#               dates affiches pour la valeur concerne (0 la plus
#               ancienne, $self->{'nbre'} - 1 la dernire)
# nbre       => le nombre d'lment du tableau "dates"
# calculator => rfrence sur un objet calculator pour le calcul des 
#               indicateurs
# indicators => rfrence sur un tableau contenant la listes des indicateurs
#                afficher
# label1 .. label7 => les labels utiliss en bas pour l'affichage des donnes
# max (min)  => plus haut (bas) cours plus (moins) 2%
# vol_max    => volume maximum
# yfac (vfac)=> facteur de proportionnalit entre pixels et cours (volume)
# zoneX_bottom => pixel du bas de la zone X (X etant 1, 2 ou 3)
# zoneX_height => hauteur en pixel de la zone X
# _wu        => unit de largeur utilise pour le dessin des chandeliers
# _rd        => diffrence relle entre max et min (pas de chgt de 2*2%)
# indic_info => un hash contenant des infos sur les indicateurs 
#               affichage auto (max, min, fac)
#
### 

#
# new : Le point de dpart de la cration de la fentre
#
sub new {
  my $self = shift;
  my $class = ref($self) || $self;
  my $main = shift;
  my $id = lc(shift);
  my $node = { 'id' => $id, 
               'db' => {}, 
               'height' => $Conf{'height'},
               'width' => $Conf{'width'},
               'width_per_date' => $Conf{'candle_width'}
             };

  my @dates = ();
  $node->{'dates'} = \@dates;

  # Prparation de la base de donnes
  tie %{$node->{'db'}}, $Conf{'db_access'}, $id, 
      split(/,/, $Conf{'db_access_options'}) or
      croak "Can't open database for id $id : $!\n";
  
  # Cration d'un calculateur d'indices associ
  $node->{'calculator'} = Calculator->new($node->{'db'});
  
  # Initialise la liste d'indicateurs
  $node->{'indicators'} = [split(',', $Conf{'indicators'})]; 

  bless $node, $class;

  # Mise  jour des donnes
  $node->update_data();

  # Cration-initialisation de la fiche
  $node->{'window'} = $main->Toplevel();
  $node->{'window'}->title($GTUtils::list{$id} . " [$id]"); 
  # Le canvas
  $node->{'canvas'} = $node->{'window'}->Scrolled('Canvas',
                      -height => $node->{'height'}, 
                      -width => $node->{'width'},
                      -background => 'black',
		      -xscrollincrement => $node->{'width_per_date'},
                      -scrollregion => [ 0, 0, $node->{'c_width'}, 
                                         $node->{'height'} ],
                      -scrollbars => 's');
  $node->{'canvas'}->xview('moveto', 1);
  $node->{'canvas'}->CanvasBind('<Any-Motion>' 
                     => [ $node, 'update_info', Ev('x'), Ev('y')]);
  $node->{'canvas'}->CanvasBind('<Configure>', [$node, 'configure']); 
  # La zone d'information du bas avec le menu
  $node->{'frame'} = $node->{'window'}->Frame();
  $node->{'frame'}->Menubutton(-text => gettext("Commands"),
                               -underline => 0, -relief => 'raised',
                               -menuitems =>
         [
	  [Button => gettext("Indicators") . "...", -command => [$node,
	  'choix_indic']],
          [Button => gettext("Export to postscript")],
          [Separator => ''],
          [Button => gettext("Close"), -command => sub {$node->DESTROY}]
         ])->pack(-side => 'left');
  foreach (1..2) {
    $node->{"label$_"} = $node->{'frame'}->Label(-text => " \n ", 
                                                 -justify => 'left', 
					         -anchor => 'w');
    $node->{"label$_"}->pack(-side => 'left');
  }
  foreach (7,6,5,4,3) {
    $node->{"label$_"} = $node->{'frame'}->Label(-text => " \n ", 
                                                 -justify => 'left', 
					         -anchor => 'ne');
    $node->{"label$_"}->pack(-side => 'right');
  }

  # Pas ncessaire car le callback configure est appel  la cration...
  #$node->draw();
  $node->{'canvas'}->pack(-expand => 1, -fill => 'both');
  $node->{'frame'}->pack(qw/-fill x/); 

  return $node;
}

#
# Mise   jour des donnes pour la valeur : calculs sommaires
# de maximums/minimums pour les cours et le volume
# Recherche des jours  dessiner en partant des plus rcents
#
sub update_data {
  my $self = shift;

  my $i = 0;
  my $time = time();
  my $ft = time_of_date($self->{'calculator'}->dates(0));
  my $ok = 0;

  # Listing the dates to draw
  while($time > $ft) {
    if (time_in_week($time) &&
        ($ok || exists $self->{'db'}->{date_of_time($time)})) {
      $ok = 1;
      push(@{$self->{'dates'}}, date_of_time($time));
      $i++;
    }
    $time -= 86400;
  }

  $self->{'nbre'} = $i;

  # Calculating the max/min/vol_max
  my($tab,$max,$min,$vol_max) = (undef, 0, -1, 0);
  foreach (@{$self->{'dates'}}) {
    next if (! exists $self->{'db'}->{$_});
    $tab = $self->{'db'}->{$_};
    $max = $tab->[1] if ($tab->[1] > $max);
    $min = $tab->[2] if ($tab->[2] < $min || $min == -1);
    $vol_max = $tab->[4] if ($tab->[4] > $vol_max);
  }

  $self->{'_rd'} = $max - $min; # Real difference beetween max and min
  $self->{'max'} = $max + $self->{'_rd'} * 0.02; 
  $self->{'min'} = $min - $self->{'_rd'} * 0.02;
  $self->{'vol_max'} = $vol_max;
  
  $self->update_calc();
}

#
# Mets  jour les valeurs calcules (facteurs...)
# en fonction des indicateurs  afficher (sparation en trois zones) !
#
sub update_calc {
  my $self = shift;
  my ($indic, $tmp);
 
  $self->{'_wu'} = int(($self->{'width_per_date'} - 1) / 3); # Width unit
  $self->{'c_width'} = $self->{'nbre'} * $self->{'width_per_date'};
  $self->{'c_width'} = $self->{'width'} if ($self->{'width'} 
                                            > $self->{'c_width'});

  # Regarde sur quelle(s) zone(s) il va y avoir des affichages
  # En dduit la taille des diffrentes zones
  my (@zones) = (0,0,0,0);
  foreach $indic (@{$self->{'indicators'}}) {
   $tmp = Indicators::get_indic_info($indic, 'representation');
   next if (! defined $tmp);
   $zones[$1] += 1 if ($tmp and $tmp =~ /_(\d)_/);
  }
  if ($zones[3] > 0) {
    $self->{'zone3_bottom'} = $self->{'height'};
    $self->{'zone3_height'} = $Conf{'zone3fac'} * $self->{'height'}; 
  } else {
    $self->{'zone3_bottom'} = 0;
    $self->{'zone3_height'} = 0;
  }
  if ($zones[2] > 0) {
    $self->{'zone2_bottom'} = $self->{'height'} - $self->{'zone3_height'};
    $self->{'zone2_height'} = $Conf{'zone2fac'} * $self->{'height'}; 
  } else {
    $self->{'zone2_bottom'} = 0;
    $self->{'zone2_height'} = 0;
  }
  if ($zones[1] > 0) {
    $self->{'zone1_bottom'} = $self->{'height'} - $self->{'zone2_height'} -
                              $self->{'zone3_height'};
    $self->{'zone1_height'} = $self->{'zone1_bottom'};
  } else {
    $self->{'zone1_bottom'} = $self->{'zone1_height'} = 0;
    # Cas spcial : la zone bouche-trou n'existe pas
    # On partage autrement
    # Zone 3 reste une petite zone et zone2 devient le bouche-trou
    if ($zones[2] > 0) {
      $self->{'zone2_bottom'} = $self->{'height'} - $self->{'zone3_height'};
      $self->{'zone2_height'} = $self->{'zone2_bottom'};
    } else {
      ## Couillonn l, il n'y a que la zone 3  afficher voire rien
      $self->{'zone3_height'} = $self->{'zone3_bottom'} = $self->{'height'};
    }
  }

  $self->{'yfac'} = $self->{'zone1_height'} / ($self->{'max'} - $self->{'min'});
  $self->{'vfac'} = $self->{'zone3_height'} / $self->{'vol_max'};

  # Calcul des max, min, fac pour l'affichage automatique de certains
  # indicateurs
  my ($max, $min, $val, $i, $val2);
  foreach $indic (grep { ($tmp = Indicators::get_indic_info($_, 'representation'))
                  =~ /^(?:(?:multi-)?courbe|histogramme)_\d_auto$/ } 
                @{$self->{'indicators'}}) {
    #La premire donne pour initialisation
    $val = $self->{'calculator'}->get($indic, $self->{'dates'}[0]);
    # Pour grer les multi-courbe et les autres
    if (ref $val) {
     $max = $min = $val->[0];
    } else {
     $max = $min = $val;
    }
    # Recherche des max/min
    for($i = 0; $i < $self->{'nbre'}; $i++) {
      $val = $self->{'calculator'}->get($indic, $self->{'dates'}[$i]);
      next if (!defined $val);
      if (ref $val) {
        foreach $val2 (@$val) {
	  $max = $val2 if ($val2 > $max);
	  $min = $val2 if ($val2 < $min);
	}
      } else {
        $max = $val if ($val > $max);
        $min = $val if ($val < $min);
      }
    }
    $self->{'indic_info'}{$indic} = { 'max' => $max, 'min' => $min,
                                      'fac' => 1 / ($max - $min) };
  }
  # On recommence pour les affichages *_perso
  foreach $indic (grep { ($tmp = Indicators::get_indic_info($_, 'representation'))
                  =~ /^(?:(?:multi-)?courbe|histogramme)_\d_perso$/ } 
                @{$self->{'indicators'}}) {
    $val = $self->{'calculator'}->get($indic, $self->{'dates'}[0]);
    if (ref $val) {
     $max = $min = $val->[0];
    } else {
     $max = $min = $val;
    }
    for($i = 1; $i < $self->{'nbre'}; $i++) {
      $val = $self->{'calculator'}->get($indic, $self->{'dates'}[$i]);
      next if (!defined $val);
      if (ref $val) {
        foreach $val2 (@$val) {
	  $max = $val2 if ($val2 > $max);
	  $min = $val2 if ($val2 < $min);
	}
      } else {
        $max = $val if ($val > $max);
        $min = $val if ($val < $min);
      }
    }
    $tmp = eval Indicators::get_indic_info($indic, 'scale')->[0];
    $min = $tmp if (! $@);
    $tmp = eval Indicators::get_indic_info($indic, 'scale')->[1];
    $max = $tmp if (! $@);
    $self->{'indic_info'}{$indic} = { 'max' => $max, 'min' => $min,
                                      'fac' => 1 / ($max - $min) };
  }
}

#
# Hash des reprsentations :
#
my (%representations) = 
   (
     'barchart_1_standard'    => \&draw_barchart_1_standard,
     'chandelier_1_standard'  => \&draw_chandelier_1_standard,
     'courbe_1_auto'          => \&draw_courbe_1_auto,
     'courbe_1_perso'         => \&draw_courbe_1_auto,
     'courbe_1_standard'      => \&draw_courbe_1_standard,
     'courbe_2_auto'          => \&draw_courbe_2_auto,
     'courbe_2_perso'         => \&draw_courbe_2_auto,
     'courbe_2_standard'      => \&draw_courbe_2_standard,
     'courbe_3_auto'          => \&draw_courbe_3_auto,
     'courbe_3_perso'         => \&draw_courbe_3_auto,
     'courbe_3_standard'      => \&draw_courbe_3_standard,
     'histogramme_1_auto'     => \&draw_histogramme_1_auto,
     'histogramme_1_perso'    => \&draw_histogramme_1_auto,
     'histogramme_1_standard' => \&draw_histogramme_1_standard,
     'histogramme_2_auto'     => \&draw_histogramme_2_auto,
     'histogramme_2_perso'    => \&draw_histogramme_2_auto,
     'histogramme_2_standard' => \&draw_histogramme_2_standard,
     'histogramme_3_auto'     => \&draw_histogramme_3_auto,
     'histogramme_3_perso'    => \&draw_histogramme_3_auto,
     'histogramme_3_standard' => \&draw_histogramme_3_standard,
     'multi-courbe_1_auto'    => \&draw_multicourbe_1_auto,
     'multi-courbe_1_perso'   => \&draw_multicourbe_1_auto,
     'multi-courbe_1_standard'=> \&draw_multicourbe_1_standard,
     'multi-courbe_2_auto'    => \&draw_multicourbe_2_auto,
     'multi-courbe_2_perso'   => \&draw_multicourbe_2_auto,
     'multi-courbe_2_standard'=> \&draw_multicourbe_2_standard,
     'multi-courbe_3_auto'    => \&draw_multicourbe_3_auto,
     'multi-courbe_3_perso'   => \&draw_multicourbe_3_auto,
     'multi-courbe_3_standard'=> \&draw_multicourbe_3_standard
   );

#
# Affichage des cours 
#

sub draw {
  my $self = shift;
  my $c = $self->{'canvas'};
  my $val;

  # Cleaning the canvas
  $self->{'canvas'}->addtag('del', 'all');
  $self->{'canvas'}->delete('del');

  # Price grid
  foreach ($self->_cut()) {
    $val = $self->_mcy($_);
    $c->createLine(0, $val, $self->{'c_width'} - 1, $val, -fill => '#868686');
    $c->createText(1, $val + 1, -anchor => 'sw', -fill => '#FFFFFF',
                   -text => $_);
  }


  # Draw all selected graphs (indicators) 
  my ($indic, $info);

  foreach $indic (sort { (Indicators::get_indic_info($a, 'order') || 0) <=>
                         (Indicators::get_indic_info($b, 'order') || 0) } 
		  @{$self->{'indicators'}}) {
    $info = Indicators::get_indic_info($indic,'representation');
    if (exists $representations{$info}) {
      &{$representations{$info}}($self, $indic);
    }
  }

}

#
# Inutile mais j'aime bien m'assurer du nettoyage
#
sub DESTROY {
  my $self = shift;

  $self->{'calculator'} = undef;
  $self->{'window'}->destroy();
  untie %{$self->{'db'}};
  %{$self} = qw();
}

#
# Choix des indicateurs  afficher
#
sub choix_indic {
  my $self = shift;
  my %indics;

  my $win = $self->{'window'}->Toplevel(-title => gettext( 
                                                       "Indicators to draw"));
  # La liste multiple
  my $l = $win->Scrolled('Listbox', 
                         -selectmode => 'multiple',
			 -width => 20,
                         -height => 15,
                         -scrollbars => 'e');
  # La listes des indicateurs disponibles
  foreach (Indicators::get_list_visible_indics(), @{$self->{'indicators'}}) {
    $indics{$_} = 0;
  }
  $l->insert(0, sort keys %indics);
  foreach (@{$self->{'indicators'}}) { $indics{$_} = 1; }
  # On slectionne ceux qui l'taient dj
  foreach (0 .. $l->size() - 1) {
     $l->selectionSet($_, $_) if ($indics{$l->get($_)});
  }
  # Le reste
  $l->pack(-expand => 'yes', -fill => 'x');
  my $f = $win->Frame();
  $f->pack;
  $f->Button(-text => gettext("OK"), -command => 
             sub {
	       @{$self->{'indicators'}} = map {$l->get($_)} ($l->curselection);
               $self->update_data(); $self->draw(); $win->destroy();
	     })->pack(-side => 'left');
  $f->Button(-text => gettext("Cancel"), 
             -command => sub { $win->destroy() })->pack(-side => 'left');
  $f->Button(-text => gettext("Apply"), -command => 
             sub {
	       @{$self->{'indicators'}} = map {$l->get($_)} ($l->curselection);
               $self->update_data(); $self->draw();
	     })->pack(-side => 'right');
}

#
# Affiche une boite d'information sur l'indicateur qui a t double-cliqu
#
sub indic_info {
  my $self = shift;
  my $indic = shift;
  
  my $win = $self->{'window'}->Toplevel(-title => $indic);
  my $text = $win->Scrolled('Text', -wrap => 'word', -scrollbars => 'e');
  $text->insert('1.0', gettext("Description") . " :\n\n" .
                   Indicators::get_indic_info($indic, 'description') .
		   "\n\n" . gettext("Explication") . " :\n\n" .
		   Indicators::get_indic_info($indic, 'explication')
		   . "\n");
  $text->pack(-expand => 'yes', -fill => 'x');
  my $f = $win->Frame();
  $f->Button(-text => gettext("Close"), 
             -command => sub { $win->destroy() })->pack;
  $f->pack;
}

#
# Fonction callback sur le mouvement de la souris => affichage d'infos
# au bas de la fiche
#
sub update_info {
  my $self = shift;

  # Calcul de l'abscisse sur le canvas complet (pas que la partie affiche)
  my $x = int(($self->{'canvas'}->Subwidget('xscrollbar')->get())[0]
          * $self->{'c_width'});
  $x += shift();
  # L'ordonne reste telle quelle
  my $y = shift() - 1;
  # On dduit de l'abscisse le jour concern
  my $ind = int(($self->{'c_width'} - $x) / $self->{'width_per_date'}); 
  return if ($ind >= $self->{'nbre'});
  my $date = $self->{'dates'}[$ind];
  my($ye,$m,$d) = $date =~ /^(\d\d\d\d)(\d\d)(\d\d)$/;
  my $fdate = $strings[7];
  $fdate =~ s/\%d/$d/; $fdate =~ s/\%m/$m/; $fdate =~ s/\%y/$ye/;
  $self->{'label1'}->configure(-text => gettext("Date") . "\n" . $fdate);

  # Diffrents cas selon les ordonnes
  if ($y < $self->{'zone1_bottom'}) {
    $self->{'label2'}->configure(-text => $strings[10] . "\n" . 
                                 sprintf('%.4g', $self->_ucy($y))); 
  } elsif ($y < $self->{'zone2_bottom'}) {
    $self->{'label2'}->configure(-text => '');
  } elsif ($y < $self->{'zone3_bottom'}) {
    $self->{'label2'}->configure(-text => $strings[5] . "\n" . 
                                 $self->_ucv($y));
  }
  # Donnes du jour, toujours affiches
  if (exists $self->{'db'}{$date}) {
    my $tab = $self->{'db'}{$date};
    foreach (3..7) {
      $self->{"label$_"}->configure(-text => $strings[$_-2] . 
                                    "\n" . $tab->[$_-3]);
    }
  } else {
    foreach (3..7) {
      $self->{"label$_"}->configure(-text => " \n ");
    }
  }
}

#
# Fonction callback pour changement de taille du canvas
#
sub configure {
  my $self = shift;

  $self->{'width'} = $self->{'canvas'}->Subwidget('canvas')->Width();
  $self->{'height'} = $self->{'canvas'}->Subwidget('canvas')->Height();

  $self->update_calc();
  $self->draw();
}

#
# draw_courbe : fonction utilise plus loins par les foncyion d'affichage
#
# Args : indic, couleur, epaisseur, bas, min, fac
# Formule de conversion : $bas - ($val - $min) * $fac
#
sub draw_courbe {
  my ($self, $indic, $color, $thick, $bottom, $min, $fac) = @_;
  my $left = int($self->{'c_width'} + $self->{'width_per_date'} / 2);
  my ($lleft, $i, $val, $pre, $cur) = (0, 0);
  my $c = $self->{'canvas'};
  $color = Indicators::get_indic_info($indic, 'color') || $color;

  for($i = 0; $i < $self->{'nbre'}; $i++) {
    $left -= $self->{'width_per_date'};
    $val = $self->{'calculator'}->get($indic, $self->{'dates'}[$i]);
    if (defined $val) {
      $cur = $bottom - ($val - $min) * $fac;
      $c->createLine($lleft + $self->{'width_per_date'}, $pre, $left,
                     $cur, -capstyle => 'round', -fill => $color, 
                     -width => $thick, -tags => [$indic]) if (defined $pre);
      $pre = $cur;
      $lleft = $left - $self->{'width_per_date'};
    }
  }
  $c->bind($indic, '<Double-ButtonPress-1>',
           [ $self, 'indic_info', $indic ] );
	  
}

#
# draw_multicourbe : fonction utilise plus loins par les fonctions d'affichage
#
# Args : indic, couleur, epaisseur, bas, min, fac
# Formule de conversion : $bas - ($val - $min) * $fac
#
sub draw_multicourbe {
  my ($self, $indic, $color, $thick, $bottom, $min, $fac) = @_;
  my $left = int($self->{'c_width'} + $self->{'width_per_date'} / 2);
  my ($lleft, $i, $j, $val, $data, @pre, @cur, @color) = (0, 0, 0);
  my $c = $self->{'canvas'};
  @color = split(/,/, Indicators::get_indic_info($indic, 'color'));
  @color = ($color) x 3 if (@color <= 1); #Au hasard trois...

  for($i = 0; $i < $self->{'nbre'}; $i++) {
    $left -= $self->{'width_per_date'};
    $val = $self->{'calculator'}->get($indic, $self->{'dates'}[$i]);
    next if (! defined $val);
    $j = 0;
    foreach $data (@{$val}) {
      $cur[$j] = $bottom - ($data - $min) * $fac;
      $c->createLine($lleft + $self->{'width_per_date'}, $pre[$j], $left,
                     $cur[$j], -capstyle => 'round', -fill => $color[$j], 
                     -width => $thick, -tags => [$indic]) if (defined $pre[$j]);
      $pre[$j] = $cur[$j];
      $j++;
    }
    $lleft = $left - $self->{'width_per_date'};
  }
  $c->bind($indic, '<Double-ButtonPress-1>',
           [ $self, 'indic_info', $indic ] );
	  
}



#
# draw_histogramme : fonction utilise plus loins pour l'affichage
#
# Args : indic, couleur, epaisseur, bas, min, fac
# Formule de conversion : $bas - ($val - $min) * $fac
#
sub draw_histogramme {
  my ($self, $indic, $color, $bottom, $min, $fac) = @_;
  my $left = $self->{'c_width'};
  my ($i, $val) = (0);
  my $c = $self->{'canvas'};
  $color = Indicators::get_indic_info($indic, 'color') || $color;

  for($i = 0; $i < $self->{'nbre'}; $i++) {
    $left -= $self->{'width_per_date'};
    $val = $self->{'calculator'}->get($indic, $self->{'dates'}[$i]);
    next if (! defined $val);
    $val = $bottom - ($val - $min) * $fac;
    $c->createRectangle($left, $bottom + $min * $fac, 
                        $left + 2 * $self->{'_wu'}, $val,
                        -fill => $color, -outline => $color, 
                        -tags => [$indic]);
  }
  $c->bind($indic, '<Double-ButtonPress-1>', 
	   [$self, 'indic_info', $indic] );
}


#
# Affichage des indicateurs par une simple courbe superpose
# aux chandeliers, les valeurs doivent tre dans le mme domaine
# que les cours de bourse. (ie cette fonction utilise le mme coefficient
# de proportionnalit que pour l'affichage des chandeliers).
#
sub draw_courbe_1_standard {  
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";

  $self->draw_courbe($indic, 'yellow', 1, $self->{'zone1_bottom'}, 
                     $self->{'min'}, $self->{'yfac'}); 
}

#
# Affiche des courbes dans la zone 1
#
sub draw_courbe_1_auto {
  my $self = shift;
  my $indic = shift;
  my $fac = $self->{'zone1_height'} * $self->{'indic_info'}{$indic}{'fac'};

  $self->draw_courbe($indic, 'blue', 1,
                     $self->{'zone1_bottom'}, 
                     $self->{'indic_info'}{$indic}{'min'}, $fac);
}
						  

#
# Affiche des courbes dans la zone 2
#
sub draw_courbe_2_standard {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone2_height'} / 100;

  $self->draw_courbe($indic, 'yellow', 1, 
                     $self->{'zone2_bottom'}, 0, $fac);
}
sub draw_courbe_2_auto {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone2_height'} * $self->{'indic_info'}{$indic}{'fac'};

  $self->draw_courbe($indic, 'blue', 1, 
                     $self->{'zone2_bottom'}, 
                     $self->{'indic_info'}{$indic}{'min'}, $fac);
}

#
# Affiche des courbes dans la zone 3
#
sub draw_courbe_3_standard {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone3_height'} / $self->{'vol_max'};

  $self->draw_courbe($indic, 'red', 1,
                     $self->{'zone3_bottom'}, 0, $fac);
}
sub draw_courbe_3_auto {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone3_height'} * $self->{'indic_info'}{$indic}{'fac'};

  $self->draw_courbe($indic, 'blue', 1,
                     $self->{'zone3_bottom'}, 
                     $self->{'indic_info'}{$indic}{'min'}, $fac);
}

#
# Histogrammes dans la zone 1
#
sub draw_histogramme_1_standard {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone1_height'} * $self->{'yfac'};

  $self->draw_histogramme($indic, 'red', $self->{'zone1_bottom'}, 
                          $self->{'min'}, $fac);
}
sub draw_histogramme_1_auto {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone1_height'} * $self->{'indic_info'}{$indic}{'fac'};

  $self->draw_histogramme($indic, 'red',
                          $self->{'zone1_bottom'},
                          $self->{'indic_info'}{$indic}{'min'}, $fac);
}

#
# Histogrammes dans la zone 2
#
sub draw_histogramme_2_standard {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone2_height'} / 100;

  $self->draw_histogramme($indic, 'red', $self->{'zone2_bottom'}, 0, $fac);
}
sub draw_histogramme_2_auto {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone2_height'} * $self->{'indic_info'}{$indic}{'fac'};

  $self->draw_histogramme($indic, 'red',
                          $self->{'zone2_bottom'},
                          $self->{'indic_info'}{$indic}{'min'}, $fac);
}

#
# Affiche des histogrammes de volume, il faut donc que l'indicateur
# renvoie des valeurs comprises entre 0 et volume_max car c'est le
# meme coeff de proportionnalit qui est utilis
#
sub draw_histogramme_3_standard {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone3_height'} / $self->{'vol_max'};

  $self->draw_histogramme($indic, 'red', $self->{'zone3_bottom'}, 0, $fac);
}

#
# Histogramme automatique dans la zone 3
#
sub draw_histogramme_3_auto {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $fac = $self->{'zone3_height'} * $self->{'indic_info'}{$indic}{'fac'};

  $self->draw_histogramme($indic, 'red',
                          $self->{'zone3_bottom'}, 
                          $self->{'indic_info'}{$indic}{'min'}, $fac);
}

#
# Affiche des chandeliers de cours, l'indicateur doit renvoyer 4 valeurs
# dans une rfrence de tableau :
# [ ouverture, plus_haut, plus_bas, cloture ]
# Ne sert que pour les cours pour le moment (donc utilise des coeffs
# calcul pour les cours).
#
sub draw_chandelier_1_standard {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $i = 0;
  my ($left,$c) = ($self->{'c_width'}, $self->{'canvas'});
  my $data;
  my $color = Indicators::get_indic_info($indic, 'color') || 'yellow';
 
  for ($i = 0; $i < $self->{'nbre'}; $i++) {
    $left -= $self->{'width_per_date'};
    $data = $self->{'calculator'}->get($indic, $self->{'dates'}[$i]);
    next if (! defined $data);
    $c->createRectangle($left, $self->_mcy($data->[0]),
                        $left + 2 * $self->{'_wu'}, $self->_mcy($data->[3]),
			-fill => ($data->[0] > $data->[3] ? $color : undef),
			-outline => $color, -tags => [$indic]);
    $c->createLine($left + $self->{'_wu'}, $self->_mcy($data->[1]),
    		   $left + $self->{'_wu'},
		   $self->_mcy($data->[0]>$data->[3] ? $data->[0]:$data->[3]),
		   -fill => $color, -tags => [$indic]);
    $c->createLine($left + $self->{'_wu'}, $self->_mcy($data->[2]),
                   $left + $self->{'_wu'},
                   $self->_mcy($data->[0]<$data->[3] ? $data->[0]:$data->[3]),
                   -fill => $color, -tags => [$indic]);
  }
  $c->bind($indic, '<Double-ButtonPress-1>',  
	   [$self, 'indic_info', $indic]);
}

#
# Graphe des cours en barchart
#
sub draw_barchart_1_standard {
  my $self = shift;
  my $indic = shift || carp "Indicateur manquant";
  my $i = 0;
  my ($left,$c) = ($self->{'c_width'}, $self->{'canvas'});
  my $data;
  my $color = Indicators::get_indic_info($indic, 'color') || 'yellow';
  my ($dec, $length) = (int(($self->{'width_per_date'}-1)/2),
                        int(($self->{'width_per_date'}-2)/2)); 
  
  for ($i = 0; $i < $self->{'nbre'}; $i++) {
    $left -= $self->{'width_per_date'};
    $data = $self->{'calculator'}->get($indic, $self->{'dates'}[$i]);
    next if (! defined $data);
    $c->createLine($left + $dec, $self->_mcy($data->[2])+1,
                   $left + $dec, $self->_mcy($data->[1]),
                    -fill => $color, -tags => [$indic]);
    $c->createLine($left + $dec, $self->_mcy($data->[0]),
                   $left + $dec - $length, $self->_mcy($data->[0]),
                    -fill => $color, -tags => [$indic]);
    $c->createLine($left + $dec + 1, $self->_mcy($data->[3]),
                   $left + $dec + $length + 1, $self->_mcy($data->[3]),
                    -fill => $color, -tags => [$indic]);
  }
  $c->bind($indic, '<Double-ButtonPress-1>',  
	   [$self, 'indic_info', $indic]);
}

#
# Affichage de plusieurs courbes  partir d'un indicateur
# multi-courbe_*
#
sub draw_multicourbe_1_standard {
  my ($self, $indic) = @_;

  $self->draw_multicourbe($indic, 'yellow', 1, $self->{'zone1_bottom'},
		          $self->{'min'}, $self->{'yfac'});
}
sub draw_multicourbe_1_auto {
  my ($self, $indic) = @_;
  my $fac = $self->{'zone1_height'} * $self->{'indic_info'}{$indic}{'fac'};
  
  $self->draw_multicourbe($indic, 'blue', 1, $self->{'zone1_bottom'},
		          $self->{'indic_info'}{$indic}{'min'}, $fac);
}
sub draw_multicourbe_2_standard {
  my ($self, $indic) = @_;
  my $fac = $self->{'zone2_height'} / 100;

  $self->draw_multicourbe($indic, 'yellow', 1, $self->{'zone2_bottom'},
		          0, $self->{'yfac'});
}
sub draw_multicourbe_2_auto {
  my ($self, $indic) = @_;
  my $fac = $self->{'zone2_height'} * $self->{'indic_info'}{$indic}{'fac'};
  
  $self->draw_multicourbe($indic, 'blue', 1, $self->{'zone2_bottom'},
		          $self->{'indic_info'}{$indic}{'min'}, $fac);
}
sub draw_multicourbe_3_standard {
  my ($self, $indic) = @_;
  my $fac = $self->{'zone3_height'} / $self->{'vol_max'};

  $self->draw_multicourbe($indic, 'red', 1, $self->{'zone3_bottom'},
		          0, $fac);
}
sub draw_multicourbe_3_auto {
  my ($self, $indic) = @_;
  my $fac = $self->{'zone3_height'} * $self->{'indic_info'}{$indic}{'fac'};
  
  $self->draw_multicourbe($indic, 'blue', 1, $self->{'zone3_bottom'},
		          $self->{'indic_info'}{$indic}{'min'}, $fac);
}

# Mapping y coordinates
sub _mcy {
 my $self = shift;
 my $val = shift;
 return  int(($self->{'zone1_bottom'} - 
             ($val - $self->{'min'}) * $self->{'yfac'}) + 0.5);
}

# Unmapping y coordinates
sub _ucy {
  my $self = shift;
  my $y = shift;
  return (($self->{'zone1_bottom'} - $y) / $self->{'yfac'} + $self->{'min'});
}

# Mapping volume coords
sub _mcv {
  my $self = shift;
  my $v = shift;

  return int(($self->{'zone3_bottom'} - $v * $self->{'vfac'}) + 0.5);
}

# Unmapping volume coordinates
sub _ucv {
  my $self = shift;
  my $y = shift;
  
  return int(($self->{'zone3_bottom'} - $y) / $self->{'vfac'});
}

#
# Cut in several 'right' value for the grid
#
sub _cut {
  my $self = shift;
  my @res = qw();
  my ($price, $int, $dif);

  my $power = log($self->{'max'} - $self->{'min'}) / log(10);
  $dif = 10**($power - int($power));
  if ($dif > 7) {
    $int = 10**(int($power)+1) / 5;
  } elsif ($dif > 4) {
    $int = 10**(int($power));
  } elsif ($dif > 1.2) {
    $int = 10**(int($power)) / 2;
  } else {
    $int = 10**(int($power)-1) * 2;
  }

  $price = (int($self->{'min'} / $int)+1) * $int;
  while( $price <= $self->{'max'} ) {
    push(@res, $price);
    $price += $int;
  }
  return @res;
}

1;
