package Indicators;

#
# The indicators system
#

use vars qw(%indics @ISA @EXPORT @EXPORT_OK);
use strict;
use Exporter;
use GTUtils;

@ISA = qw(Exporter);
@EXPORT = qw(register_indic);
@EXPORT_OK = qw(get_list_indics get_indic_info);

## The contents of the value is a hash with :
## - code => a reference on a scalar containing the code for calculating
##           the indicator
## - param => parameters that are to be passed to the calcul sub
## - description => obvious
## - explication => obvious
## - representation => how to show the indicator (curb, bar chart, etc.)
## - range => [ -13, 0 ] => the days that must be available
## - color => the color used to draw the indicator
## - visible => a boolean that indicate if the indicator should be visible
## - nocache => will the result be cached ?
## - order => -1 for an indicator that has to be drawn first (under the
##            other) or 1 for an indicator that has to be drawn last (over
##            the others). Don't fill 'order' if you don't need it.

sub register_indic {
  my $indic = shift; 
  $indics{$indic} = { @_ };
}

sub get_list_indics {
  return (sort keys %indics);
}

sub get_list_visible_indics {
  return (sort grep { $indics{$_}{'visible'} } keys %indics);
}

#
# Access the information of an available indicator
#
sub get_indic_info {
  my ($indic, $info) = @_; 
  my(%i, $cle, $inf, $res, @parts, $indic_spe);
  #
  # Si l'indic n'existe pas, essaie de le traiter comme un indic
  # spcial  arguments de la forme /indic(-#)+/
  #
  if (! exists $indics{$indic}) {
    @parts = split(/-/,$indic);
    return undef if (scalar(@parts) <= 1);
    $indic_spe = $parts[0] . '-#' x (scalar(@parts) - 1);
    return undef if (!exists $indics{$indic_spe});
    # Clone the special hash
    %i = hash_clone($indics{$indic_spe});
    # Remplacement des %1 et autres
    foreach $cle (keys %i) {
      $_ = ref($i{$cle});
      if (m/ARRAY/) { foreach (@{$i{$cle}}) { s/%(\d+)/$parts[$1]/g } }
      if (m/HASH/)  { foreach (%{$i{$cle}}) { s/%(\d+)/$parts[$1]/g } }
      if (not $_)   { $i{$cle} =~ s/%(\d+)/$parts[$1]/g; }
    }
    # Eval les formules ...
    foreach (@{$i{'range'}}, @{$i{'param'}}) {
      $res = eval $_; 
      if (! $@) { $_ = $res }
    }
    # Stocke le rsultat
    $indics{$indic} = \%i;
  }
  # Indicateur normal
  if ($info) {
    return $indics{$indic}{$info};
  } else {
    return $indics{$indic};
  }
}

#
# Clone le hash sur un niveau de profondeur
#
sub hash_clone {
 my $ref = shift;
 my (%hash, $type);
 foreach (keys %$ref) {
   $type = ref($ref->{$_});
   if ($type =~ /ARRAY/)  { $hash{$_} = [ @{$ref->{$_}} ]; }
   if ($type =~ /HASH/)   { $hash{$_} = { %{$ref->{$_}} }; }
   if ($type =~ /SCALAR/) { $hash{$_} = \"$ref->{$_}"; }
   if (not $type)         { $hash{$_} = $ref->{$_}; } 
 }
 # Corrige les dfauts du clonage complet
 $hash{'code'} = $ref->{'code'};
 return %hash;
}

## The list of indicators :

use Indicators::COURS;
use Indicators::VOLUME;
use Indicators::AMA;
use Indicators::TMA;
use Indicators::EMA;
use Indicators::WMA;
use Indicators::MOMP;
use Indicators::ROCC;
use Indicators::BOL;


1;
